#!/usr/bin/python3

# Script generates gcode for a winding machine.
# Details on the project page:
# https://homofaciens.de/technics-machines-Winding-Machine_en.htm

# makes use of python serial. Install with:
# sudo apt-get install python3-serial


import os      # needed for file operations
import sys     # Needed for handling command line parameters
import serial
import time

printer = ""

# Parameter section ####################################################

# Set feedrate for movements
feedRate = " F18000.000"

# Diameter of the inductor wire
wire_d = 0.1

# Length of the spool
spool_l = 9.7

# Bottom Z position
bottom_Z = 5.0

top_Z = bottom_Z + spool_l # Don't change this line

# Start position of X axis (currently not used, so leave at 0.0)
X_start = 0.0

# Start position of Y axis (move spool closer to the wire guide
Y_start = 55.0

# Set mm of Extruder movement for a full turn
E_per_turn = 32.41

# Extruder direction 1 or -1
E_dir = 1

# Don't change this line
E_per_layer = E_dir * E_per_turn * spool_l / wire_d

# Number of layers to wind
Layers = 6

# Set if data is written to File ( = 1) or USB ( = 0)
File_Output = 1

# Set the accordant USB port if printer is to be controlled via USB
port='/dev/ttyUSB0', #Replace ttyS0 with ttyACM0 for Pi1,Pi2,Pi0
# port='/dev/ttyACM0', #Replace ttyACM0 with ttyUSB0 for Pi3 and Pi4

# Only make changes if you know what to do
FileName = "Wind_S_" + "{:.3f}".format(spool_l) + "_W_" + "{:.3f}".format(wire_d) + "_L_" + "{:.0f}".format(Layers) + ".gcode"

# End of Parameter section #############################################


#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Sub open USB (printer)
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
def open_USB():
    # Open USB port to Arduino running printer
    printer = serial.Serial(
    port='/dev/ttyUSB0', #Replace ttyS0 with ttyACM0 for Pi1,Pi2,Pi0
    # port='/dev/ttyACM0', #Replace ttyACM0 with ttyUSB0 for Pi3 and Pi4
    baudrate = 115200,
    parity=serial.PARITY_NONE,
    stopbits=serial.STOPBITS_ONE,
    bytesize=serial.EIGHTBITS,
    timeout=1
    )
    #time.sleep(1.002)# Give port some time to really open...
    print ("Initializing printer...")
    printer.write("$\r\n".encode('utf-8'))
    read_temp = printer.readline().strip() # Wait for printer response
    while read_temp != b'':
        print(read_temp) # Debug response
        read_temp = printer.readline().strip() # Wait for printer response

    printer.flushInput()
    time.sleep(2)
    return printer

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Sub write line to printer
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
def write2printer(printer, command_line = "", wait_for_idle = 1):

    # Remove comments
    if command_line.find(";") > 0:
        command_line = command_line[0:command_line.find(";")]

    if File_Output:
        write2gcodeFile(command_line)
        return 0
    # Empty read buffer
    #printer.flushInput()

    while(printer.read() != b''):
        time.sleep(0.002)


    print("Writing command: ", command_line)
    command_line = command_line + '\r' + '\n'
    printer.write(command_line.encode('utf-8'))# Write line to USB

    if(wait_for_idle == 2):
        printer_line = b""
        while(printer_line.find(b"OK") == -1):
          # print(printer_line)
          printer_line = b""
          read_from_cnc = printer.read(1)
          while(read_from_cnc.upper() != b'\n'):
            if(read_from_cnc != b'\r'):
              printer_line = printer_line + read_from_cnc.upper()
            read_from_cnc = printer.read(1)
          time.sleep(0.002)
          print("printer_line", printer_line)


    #print ("Command " + str(command_line) + ": " + str(printer_line) + " from Arduino")

    if(wait_for_idle == 1):
      printer_line = b""
      while(printer_line.find(b"IDLE") == -1):
        print(printer_line)
        printer_line = b""
        read_from_cnc = printer.read(1)
        if (read_from_cnc == b''):
          printer.write(b"?\n")# Request status report
        while(read_from_cnc.upper() != b'\n'):
          if(read_from_cnc != b'\r'):
            printer_line = printer_line + read_from_cnc.upper()
          read_from_cnc = printer.read(1)
        time.sleep(0.002)

    # Empty read buffer
    #printer.flushInput()

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Sub write line to gcode file
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
def write2gcodeFile(command_line = ""):

  command_line = command_line + '\r' + '\n'
  gcode_file.write(command_line)



########################################################################
########################################################################
########################################################################

if not File_Output:
    printer = open_USB()
else:
    gcode_file = open(FileName, "w")


write2printer(printer, "G21", 2) # Use millimeters
write2printer(printer, "M203 E1500", 2) # Set maximum feedrate Extruder
write2printer(printer, "M201 E5", 2) # Set maximum acceleration Extruder
write2printer(printer, "M203 Y10", 2) # Set maximum feedrate Y axis
write2printer(printer, "M203 Z500", 2) # Set maximum feedrate Z axis
write2printer(printer, "M302 P1", 2) # Enable cold extrusion
write2printer(printer, "M302 S1 R1)", 2) # Enable cold extrusion, 2nd version (S=Extrude temp, R=Retract temp)
write2printer(printer, "G28 X", 2) # Home X axis
write2printer(printer, "G28 Y", 2) # Home Y axis
write2printer(printer, "G28 Z", 2) # Home Z axis
write2printer(printer, "G92 E0", 2) # Set Extruder to 0

write2printer(printer, "G1 Z" + "{:.3f}".format(bottom_Z) + " " + feedRate, 2) # Bottom Z point
write2printer(printer, "G1 X" + "{:.3f}".format(X_start) + " Y" + "{:.3f}".format(Y_start)+ " " + feedRate, 2) # X and Y to starting point

Z_bottom = "{:.3f}".format(bottom_Z)
Z_top =  "{:.3f}".format(bottom_Z + spool_l)
E_length = "{:.3f}".format(E_per_layer)



#write2printer(printer, "G1 E" + "{:.3f}".format(E_per_turn * 10) + " " + feedRate, 2) # Test Extrusion

L = 0
while L < Layers:
    write2printer(printer, "G1 Z" + Z_top + " E" + E_length + feedRate, 2) # Top Z point
    write2printer(printer, "G92 E0", 2) # Set Extruder to 0
    L += 1
    if L < Layers:
        write2printer(printer, "G1 Z" + Z_bottom + " E" + E_length + feedRate, 2) # Top Z point
        write2printer(printer, "G92 E0", 2) # Set Extruder to 0
    L += 1


write2printer(printer, "M84", 0) # Disable motors
write2printer(printer, "M2", 0) # End of Gcode programm

print("Job done!")

if not File_Output:
    printer.close()
else:
    gcode_file.close()

exit(0)


